import { Component, OnInit } from '@angular/core';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Observable } from 'rxjs/Observable';
import { FileElement } from '../file-explorer/model/element';
import { TesService } from '../tes.service';
import { ComputesitesService } from '../computesites.service';
import { AuthorisationService } from '../authorisation.service';
import { Computesite } from '../computesite';
import { Identity } from '../identity';
import { Router } from '@angular/router';





@Component({
  selector: 'app-transfer',
  templateUrl: './transfer.component.html',
  styleUrls: ['./transfer.component.css']
})
export class TransferComponent implements OnInit {

  fileElements: BehaviorSubject<FileElement[]>[];
  pathSubject: BehaviorSubject<string>[];
  path: string[];
  working: BehaviorSubject<Boolean>[];
  canNavigateUp: BehaviorSubject<Boolean>[];
  id: Identity[];
  site: Computesite[];
  disableSend: Boolean;
  statusMsg: BehaviorSubject<string>;
  redirecturi: string;
  appevent: any;


  constructor(private computesitesService: ComputesitesService,
              private tesService: TesService,
              private authService: AuthorisationService,
              private router: Router) {
    this.path = ['',''];
    this.fileElements = [new BehaviorSubject([]), new BehaviorSubject([])];
    this.working = [ new BehaviorSubject<Boolean>(false), new BehaviorSubject<Boolean>(false)];
    this.canNavigateUp = [ new BehaviorSubject<Boolean>(false), new BehaviorSubject<Boolean>(false)];
    this.pathSubject = [ new BehaviorSubject(""),new BehaviorSubject("")];
    this.id = [null,null];
    this.disableSend = true;
    this.statusMsg = this.tesService.statusMsg;
  }

  ngOnInit() {
    var fel: FileElement[] = []
    // this.fileElements = [new BehaviorSubject([]), new BehaviorSubject([])];
    // this.working = [ new BehaviorSubject<Boolean>(false), new BehaviorSubject<Boolean>(false)];
    // this.canNavigateUp = [ new BehaviorSubject<Boolean>(false), new BehaviorSubject<Boolean>(false)];
    // this.pathSubject = [ new BehaviorSubject(""),new BehaviorSubject("")];
    // this.id = [null,null];
    // this.disableSend = true;
    setTimeout( () => this.authService.updateAgentContents() )
  }

compareObjects(o1: any, o2: any) {
  if(o1.name == o2.name) {
    return true;
  } else {
    return false;
  }
}

processError(idx,error) {
  this.working[idx].next(false);
  this.statusMsg.next(error.error.message);

}

newDirectory(idx: number, name: string) {
  this.working[idx].next(true);
  this.tesService.postMkDir(this.id[idx],this.path[idx],name).subscribe(resp => this.getSftpls(idx,'.'), error => this.processError(idx,error));
}

getSftpls(idx,cd) {
  this.working[idx].next(true);
  this.tesService.getSftpData(this.id[idx],this.path[idx],cd).subscribe(resp => this.updateSftpls(idx,resp), error => this.processError(idx,error));
}

makeFileElements(data) {
  var rv: FileElement[] = [];
  var fe: FileElement;
  for (let f of data) {
    fe = new FileElement();
    if (f.isdir !== undefined) {
      fe.isFolder = f.isdir;
    } else {
      if (f.mode.indexOf('d')== 0) {
        fe.isFolder=true;
      }
    }
    fe.name = f.name;
    fe.size = f.size;
    fe.online = f.online;
    rv.push(fe);
  }
  return rv;
}

 updateSftpls(idx,resp) {
   let files = this.makeFileElements(resp.files);
   if (resp.pwd.length > 1) {
     this.canNavigateUp[idx].next(true);
   } else {
     this.canNavigateUp[idx].next(false);

   }
   this.path[idx] = resp.pwd;
   this.pathSubject[idx].next(this.path[idx]);
   this.fileElements[idx].next(files);
   this.working[idx].next(false);

 }


  setId(idx,event) {
    this.id[idx] = event.value;
    if ((this.id[0] != null) && (this.id[1] != null)) {
      this.disableSend = false;
    }
    this.path[idx]=".";
    this.pathSubject[idx].next(this.id[idx].username+"@"+this.id[idx].site.name+":"+this.path[idx]);
    this.getSftpls(idx,'.');

  }

  navLaunch() {
    this.router.navigate(['/launch']);
  }

  navigateToFolder(idx,element: FileElement) {
    this.getSftpls(idx,element.name);
  }

  navigateUp(idx: number, event: any) {
    this.getSftpls(idx,'..');
  }



  sendFile(idx: number, element: FileElement) {
    let src = idx;
    let dest = (idx+1) % 2;
    this.id[src].keyCerts = this.authService.getKeys(this.id[src]);
    this.id[dest].keyCerts = this.authService.getKeys(this.id[dest]);
    if ((this.id[src] != null) && (this.id[dest] != null)) {
      let appparams = JSON.stringify(
        {'src': {
          'id': this.id[src],
          'path': this.path[src],
          'file': element.name },
        'dest': {
          'id': this.id[dest],
          'path': this.path[dest]
        }});
      window.parent.postMessage({'appData':appparams},'*');
    }
  }

}
